"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const xjs = require("extrajs");
const Node_class_1 = require("./Node.class");
class xjs_Element extends Node_class_1.default {
    constructor(node) {
        super(node);
    }
    get node() { return super.node; }
    get tagName() { return this.node.tagName.toLowerCase(); }
    innerHTML(markup) {
        console.warn(`WARNING: Setting \`innerHTML\` is a security risk. Use \`.textContent\` or \`.append\` instead.`);
        if (!arguments.length)
            return this.node.innerHTML;
        this.node.innerHTML = markup;
        return this;
    }
    outerHTML(markup) {
        if (!arguments.length)
            return this.node.outerHTML;
        throw new Error('feature not supported yet');
    }
    prepend(...contents) {
        this.node.prepend(...contents.map((c) => (c instanceof Node_class_1.default) ? c.node :
            (c === null) ? '' : c));
        return this;
    }
    append(...contents) {
        this.node.append(...contents.map((c) => (c instanceof Node_class_1.default) ? c.node :
            (c === null) ? '' : c));
        return this;
    }
    querySelector(selector) {
        let el = this.node.querySelector(selector);
        return (el === null) ? null : new xjs_Element(el);
    }
    querySelectorAll(selector) {
        return [...this.node.querySelectorAll(selector)].map((el) => new xjs_Element(el));
    }
    before(...contents) {
        this.node.before(...contents.map((c) => (c instanceof Node_class_1.default) ? c.node :
            (c === null) ? '' : c));
        return this;
    }
    after(...contents) {
        this.node.after(...contents.map((c) => (c instanceof Node_class_1.default) ? c.node :
            (c === null) ? '' : c));
        return this;
    }
    attr(attr, value, this_arg = this) {
        return xjs.Object.switch(xjs.Object.typeOf(attr), {
            'object': (atr) => {
                for (let i in atr)
                    this.attr(i, atr[i]);
                return this;
            },
            'string': (atr) => {
                if (atr.trim() === '')
                    throw new RangeError('Attribute name cannot be empty string.');
                return xjs.Object.switch(xjs.Object.typeOf(value), {
                    'function': (val) => this.attr(atr, val.call(this_arg)),
                    'string': (val) => (this.node.setAttribute(atr, val), this),
                    'number': (val) => (this.node.setAttribute(atr, val.toString()), this),
                    'infinite': (val) => (this.node.setAttribute(atr, val.toString()), this),
                    'boolean': (val) => (this.node.setAttribute(atr, val.toString()), this),
                    'null': () => (this.node.removeAttribute(atr), this),
                    'undefined': () => this.node.getAttribute(atr),
                    'NaN': (val) => { throw xjs.Number.assertType(val); },
                })(value);
            },
            'null': () => this,
            'undefined': () => this,
        })(attr);
    }
    id(value, this_arg = this) {
        if (!arguments.length)
            return this.node.id;
        if (typeof value === 'string')
            this.node.id = value;
        else
            this.attr('id', value, this_arg);
        return this;
    }
    class(value, this_arg = this) {
        if (!arguments.length)
            return this.node.className;
        if (typeof value === 'string')
            this.node.className = value;
        else
            this.attr('class', value, this_arg);
        return this;
    }
    addClass(...tokens) {
        tokens.forEach((token) => {
            token.split(' ').forEach((t) => {
                if (t.trim() !== '')
                    this.node.classList.add(t);
            });
        });
        return this;
    }
    removeClass(...tokens) {
        tokens.forEach((token) => {
            token.split(' ').forEach((t) => {
                if (t.trim() !== '')
                    this.node.classList.remove(t);
            });
        });
        return this;
    }
    replaceClassString(old_, new_) {
        this.node.className = this.node.className.replace(old_, new_);
        return this;
    }
}
exports.default = xjs_Element;
