"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const util = require("util");
const template_processor_1 = require("template-processor");
const DocumentFragment_class_1 = require("./DocumentFragment.class");
const HTMLElement_class_1 = require("./HTMLElement.class");
class xjs_HTMLTemplateElement extends HTMLElement_class_1.default {
    static fromString(str) {
        let elem = DocumentFragment_class_1.default.fromString(str).node.querySelector('template');
        if (elem === null) {
            throw new ReferenceError(`No \`template\` element was found.`);
        }
        return new xjs_HTMLTemplateElement(elem);
    }
    static async fromFile(filepath) {
        let str = await util.promisify(fs.readFile)(filepath, 'utf8');
        try {
            return xjs_HTMLTemplateElement.fromString(str);
        }
        catch (_a) {
            throw new ReferenceError(`No \`template\` element was found in file: \`${filepath}\`.`);
        }
    }
    static fromFileSync(filepath) {
        let str = fs.readFileSync(filepath, 'utf8');
        try {
            return xjs_HTMLTemplateElement.fromString(str);
        }
        catch (_a) {
            throw new ReferenceError(`No \`template\` element was found in file: \`${filepath}\`.`);
        }
    }
    constructor(node) {
        super(node);
    }
    get node() { return super.node; }
    content() {
        return this.node.content;
    }
    render(renderer, data, options = {}, this_arg = this) {
        return new template_processor_1.Processor(this.node, renderer).process(data, options, this_arg);
    }
}
exports.default = xjs_HTMLTemplateElement;
